import { __decorate, __extends, __metadata } from "tslib";
import { Component, Prop } from 'vue-property-decorator';
import BaseComponent from '../BaseComponent';
import { Icon } from '../Icon';
import { Label } from '../Label';
import { Callout } from '../Callout';
import { ActionButton } from '../Button';
import { Checkbox } from '../Checkbox';
import { classNamesFunction } from '@uifabric-vue/utilities';
var getClassNames = classNamesFunction();
var DropdownBase = /** @class */ (function (_super) {
    __extends(DropdownBase, _super);
    function DropdownBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.isOpen = false;
        _this.calloutRenderEdge = undefined;
        return _this;
    }
    DropdownBase.prototype.created = function () {
        var _this = this;
        this.options.forEach(function (option) {
            _this.$set(option, 'isItemSelected', false);
        });
    };
    Object.defineProperty(DropdownBase.prototype, "classNames", {
        get: function () {
            var _a = this, theme = _a.theme, className = _a.className, errorMessage = _a.errorMessage, label = _a.label, required = _a.required, disabled = _a.disabled, panelProps = _a.panelProps, calloutProps = _a.calloutProps;
            var _b = this, isOpen = _b.isOpen, calloutRenderEdge = _b.calloutRenderEdge;
            var selectedOptions = this.selectedOptions;
            return getClassNames(this.styles, {
                theme: theme,
                className: className,
                hasError: !!(errorMessage && errorMessage.length > 0),
                hasLabel: !!label,
                isOpen: isOpen,
                required: required,
                disabled: disabled,
                isRenderingPlaceholder: !selectedOptions.length,
                panelClassName: panelProps === null || panelProps === void 0 ? void 0 : panelProps.className,
                calloutClassName: calloutProps === null || calloutProps === void 0 ? void 0 : calloutProps.className,
                calloutRenderEdge: calloutRenderEdge,
            });
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DropdownBase.prototype, "multiSelectItemStyles", {
        get: function () {
            return this.classNames.subComponentStyles
                ? this.classNames.subComponentStyles.multiSelectItem
                : undefined;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DropdownBase.prototype, "hasErrorMessage", {
        get: function () {
            return this.errorMessage && this.errorMessage.length > 0;
        },
        enumerable: false,
        configurable: true
    });
    DropdownBase.prototype.select = function (option) {
        if (option.disabled)
            return;
        var index = this.selectedOptions.findIndex(function (o) { return o.key === option.key; });
        if (index > -1) {
            this.selectedOptions[index].isItemSelected = false;
            this.selectedOptions.splice(index, 1);
        }
        else {
            if (!this.multiSelect) {
                this.selectedOptions.splice(0, this.selectedOptions.length);
                this.isOpen = false;
            }
            option.isItemSelected = true;
            this.selectedOptions.push(option);
        }
    };
    DropdownBase.prototype.onPositioned = function (positions) {
        if (!this.calloutRenderEdge || this.calloutRenderEdge !== positions.targetEdge) {
            this.calloutRenderEdge = positions.targetEdge;
        }
    };
    DropdownBase.prototype.render = function () {
        var _this = this;
        var _a = this, classNames = _a.classNames, label = _a.label, selectedOptions = _a.selectedOptions, multiSelect = _a.multiSelect, multiSelectDelimiter = _a.multiSelectDelimiter, placeholder = _a.placeholder, isOpen = _a.isOpen, dropdownWidth = _a.dropdownWidth, options = _a.options, hasErrorMessage = _a.hasErrorMessage, errorMessage = _a.errorMessage, required = _a.required, disabled = _a.disabled;
        var OptionComponent = multiSelect ? Checkbox : ActionButton;
        return (<div class={classNames.root}>
        <Label class={classNames.label} required={required} disabled={disabled}>
          {label}
        </Label>
        <div ref="dropdown" class={classNames.dropdown} onClick={function () { return (_this.isOpen = true); }}>
          <span class={classNames.title}>
            {selectedOptions.length
            ? selectedOptions.map(function (i) { return i.text; }).join(multiSelectDelimiter)
            : placeholder}
          </span>
          <span class={classNames.caretDownWrapper}>
            <Icon class={classNames.caretDown} icon-name="ChevronDown"/>
          </span>
        </div>

        {isOpen && (<div>
            <Callout target={this.$refs.dropdown} is-beak-visible={false} callout-width={dropdownWidth || (this.$refs.dropdown ? this.$refs.dropdown.clientWidth : 0)} onDismiss={function () { return (_this.isOpen = false); }} onPositioned={this.onPositioned}>
              <div class={classNames.dropdownItemsWrapper} tabindex={0}>
                {options.map(function (option, index) { return (<OptionComponent key={index} class={option.hidden
            ? classNames.dropdownItemHidden
            : option.isItemSelected && option.disabled === true
                ? classNames.dropdownItemSelectedAndDisabled
                : option.isItemSelected
                    ? classNames.dropdownItemSelected
                    : option.disabled === true
                        ? classNames.dropdownItemDisabled
                        : classNames.dropdownItem} disabled={option.disabled} title={option.text} checked={option.isItemSelected} styles={multiSelect ? _this.multiSelectItemStyles : null} role={option} nativeOnClick={function () { return (!multiSelect && _this.select(option)); }} onInput={function () { return (multiSelect && _this.select(option)); }}>
                    {option.text}
                  </OptionComponent>); })}
              </div>
            </Callout>
          </div>)}

        {hasErrorMessage && (<div role="alert" class={classNames.errorMessage}>
            {errorMessage}
          </div>)}
      </div>);
    };
    __decorate([
        Prop({ type: Array, required: true }),
        __metadata("design:type", Array)
    ], DropdownBase.prototype, "options", void 0);
    __decorate([
        Prop({ type: Array, default: function () { return []; } }),
        __metadata("design:type", Array)
    ], DropdownBase.prototype, "selectedOptions", void 0);
    __decorate([
        Prop({ type: String, default: '' }),
        __metadata("design:type", String)
    ], DropdownBase.prototype, "label", void 0);
    __decorate([
        Prop({ type: String, default: '' }),
        __metadata("design:type", String)
    ], DropdownBase.prototype, "placeholder", void 0);
    __decorate([
        Prop({ type: String, default: '' }),
        __metadata("design:type", String)
    ], DropdownBase.prototype, "errorMessage", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], DropdownBase.prototype, "required", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], DropdownBase.prototype, "disabled", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], DropdownBase.prototype, "multiSelect", void 0);
    __decorate([
        Prop({ type: String, default: ', ' }),
        __metadata("design:type", String)
    ], DropdownBase.prototype, "multiSelectDelimiter", void 0);
    __decorate([
        Prop({ type: Number, default: 0 }),
        __metadata("design:type", Number)
    ], DropdownBase.prototype, "dropdownWidth", void 0);
    __decorate([
        Prop({ type: Object, default: function () { } }),
        __metadata("design:type", Object)
    ], DropdownBase.prototype, "panelProps", void 0);
    __decorate([
        Prop({ type: Object, default: function () { } }),
        __metadata("design:type", Object)
    ], DropdownBase.prototype, "calloutProps", void 0);
    DropdownBase = __decorate([
        Component({
            components: { Callout: Callout, ActionButton: ActionButton, Checkbox: Checkbox, Icon: Icon, Label: Label },
        })
    ], DropdownBase);
    return DropdownBase;
}(BaseComponent));
export { DropdownBase };
//# sourceMappingURL=Dropdown.base.jsx.map