import { ref, watchEffect } from '@vue/composition-api';
import { useConst } from './useConst';
export function useControllableValue(controlledValue, defaultUncontrolledValue, onChange) {
    var value = ref(defaultUncontrolledValue);
    var isControlled = useConst(controlledValue !== undefined);
    var currentValue = isControlled ? controlledValue : value.value;
    // Duplicate the current value and onChange in refs so they're accessible from
    // setValueOrCallOnChange without creating a new callback every time
    var valueRef = ref(currentValue);
    var onChangeRef = ref(onChange);
    watchEffect(function () {
        valueRef.value = currentValue;
        onChangeRef.value = onChange;
    });
    // To match the behavior of the setter returned by React.useState, this callback's identity
    // should never change. This means it MUST NOT directly reference variables that can change.
    var setValueOrCallOnChange = useConst(function () { return function (update, ev) {
        // Assuming here that TValue is not a function, because a controllable value will typically
        // be something a user can enter as input
        var newValue = typeof update === 'function' ? update(valueRef.value) : update;
        if (onChangeRef.value) {
            onChangeRef.value(ev, newValue);
        }
        if (!isControlled) {
            value.value = newValue;
        }
    }; });
    return [currentValue, setValueOrCallOnChange];
}
//# sourceMappingURL=useControllable.js.map